/* $Header: d:/cvsroot/tads/html/htmlpng.h,v 1.2 1999/05/17 02:52:21 MJRoberts Exp $ */

/* Copyright (c) 1997 by Michael J. Roberts.  All Rights Reserved. */
/*
Name
  htmlpng.h - PNG image object
Function
  
Notes
  
Modified
  11/08/97 MJRoberts  - Creation
*/

#ifndef HTMLPNG_H
#define HTMLPNG_H

/* include PNG library header */
#include <png.h>

#ifndef TADSHTML_H
#include "tadshtml.h"
#endif

/*
 *   Options flags 
 */

/*
 *   Keep alpha information.  If this flag isn't set, we'll use a fixed
 *   background color (obtained from the image, if possible) for images
 *   with transparency information.  If this flag is set, we'll retain
 *   alpha information, either by expanding all images to 32 bits per
 *   pixel RGBA format by default, or keeping a separate alpha mask for
 *   paletted images when HTMLPNG_OPT_PAL_ALPHA is set.  
 */
#define HTMLPNG_OPT_KEEP_ALPHA  0x0001


class CHtmlPng
{
public:
    CHtmlPng();
    ~CHtmlPng();

    /* load an image from a file */
    int read_png_file(const textchar_t *filename,
                      unsigned long seekpos, unsigned long filesize,
                      int convert_256_color);

    /* get the palette - returns null for RGB images */
    HTML_color_t *get_palette() const { return palette_; }
    unsigned int get_palette_size() const { return palette_size_; }

    /* get the dimensions */
    unsigned long get_height() const { return height_; }
    unsigned long get_width() const { return width_; }

    /* get the number of bytes per row */
    unsigned long get_row_bytes() const { return row_bytes_; }

    /* get the row pointers */
    const png_byte *const *get_rows() const { return rows_; }

    /* 
     *   Get the alpha row pointers - this returns an array of pointers to
     *   rows of bytes, with one byte of alpha per pixel.  This will be
     *   null unless the image was read in convert_256_color mode (so the
     *   image in memory is paletted), AND the HTMLPNG_OPT_KEEP_ALPHA flag
     *   is set.
     *   
     *   If the image in memory is full color and has transparency
     *   information, it will be stored with 32 bits per pixel in BGRA
     *   format, so the alpha channel must be retrieved directly from the
     *   get_rows() for this in-memory format.  
     */
    const png_byte *const *get_alpha() const { return alpha_; }

    /* color depth in bits per pixel */
    int get_bits_per_pixel() const { return bpp_; }

    /* set global options (HTMLPNG_OPT_xxx flags) */
    static void set_options(unsigned int options)
    {
        /* remember the options */
        options_ = options;
    }

private:
    /* allocate memory for the image */
    void allocate_rows(png_struct *png_ptr, png_info *info_ptr,
                       unsigned long height, unsigned long width,
                       int sep_alpha);
    
    /* delete the existing image, if any */
    void delete_image();

    /* delete the existing alpha channel information, if any */
    void delete_alpha();

    /* PNG callback function to read from the file */
    static void fread_cb(png_structp, png_bytep, png_size_t);

    /* palette, if the image has been converted to a palette-index format */
    HTML_color_t *palette_;
    unsigned int palette_size_;
    
    /* pointers to the rows containing the image bytes */
    png_bytep *rows_;

    /* pointers to the rows containing the alpha bytes */
    png_bytep *alpha_;

    /* number of rows allocated */
    unsigned long height_;

    /* width of each row */
    unsigned long width_;

    /* bytes in each row */
    unsigned long row_bytes_;

    /* number of bits per pixel */
    int bpp_;

    /* global options */
    static unsigned int options_;
};

#endif /* HTMLPNG_H */

